/*
 * include/asm-arm/arch-ixp2000/ixdp2801.h
 *
 * Register and other defines for IXDP2801
 *
 * Based on: include/asm-arm/arch-ixp2000/ixdp2400.h, by Naeem Afzal <naeem.m.afzal@intel.com>
 *
 * Copyright (c) 2002-2003 Intel Corp.
 *
 *  This program is free software; you can redistribute  it and/or modify it
 *  under  the terms of  the GNU General  Public License as published by the
 *  Free Software Foundation;  either version 2 of the  License, or (at your
 *  option) any later version.
 *
 * Changelog:
 *  07-May-2003: Andrzej Mialkowski <andrzej.mialkowski@intel.com>
 *     - Adopted from IXDP2401, updated to reflect IXDP2801 memory map
 */

#ifndef _IXDP2801_H_
#define _IXDP2801_H_


/*
 * On board CPLD memory map
 */
#define IXDP2801_PHY_SP_BASE	0xc6000000
#define IXDP2801_VIRT_SP_BASE	0xd4000000
#define IXDP2801_SP_SIZE	0x01000000

#define IXDP2801_VIRT_NVRAM_BASE	(IXDP2801_VIRT_SP_BASE + 0x000000)
#define IXDP2801_VIRT_MB_IXF1104_BASE	(IXDP2801_VIRT_SP_BASE + 0x020000)
#define IXDP2801_VIRT_CPLD_BASE		(IXDP2801_VIRT_SP_BASE + 0x024000)
#define IXDP2801_VIRT_ADD_UART_BASE	(IXDP2801_VIRT_SP_BASE + 0x024040)
#define IXDP2801_VIRT_SAUS_UART_BASE	(IXDP2801_VIRT_SP_BASE + 0x024060)
#define IXDP2801_VIRT_CS8900_BASE	(IXDP2801_VIRT_SP_BASE + 0x024080)
#define IXDP2801_VIRT_FORK_BASE		(IXDP2801_VIRT_SP_BASE + 0x0240A0)
#define IXDP2801_VIRT_RIVERDALE_BASE	(IXDP2801_VIRT_SP_BASE + 0x0240C0)
#define IXDP2801_VIRT_COMET_BASE	(IXDP2801_VIRT_SP_BASE + 0x024800)
#define IXDP2801_VIRT_ZARLINK_BASE	(IXDP2801_VIRT_SP_BASE + 0x025000)
#define IXDP2801_VIRT_FIC_BASE		(IXDP2801_VIRT_SP_BASE + 0x200000)
#define IXDP2801_VIRT_DB0_BASE		(IXDP2801_VIRT_SP_BASE + 0x400000)
#define IXDP2801_VIRT_DB1_BASE		(IXDP2801_VIRT_SP_BASE + 0x600000)

#define IXDP2801_CPLD_REG(reg) (volatile unsigned long *)(IXDP2801_VIRT_CPLD_BASE | reg)

#define IXDP2801_CPLD_RESET_REG		IXDP2801_CPLD_REG(0x00)
#define IXDP2801_CPLD_CLOCK_REG		IXDP2801_CPLD_REG(0x04)
#define IXDP2801_CPLD_INT_MASK_SET_REG	IXDP2801_CPLD_REG(0x08)
#define IXDP2801_CPLD_INT_STAT_REG	IXDP2801_CPLD_REG(0x0C)
#define IXDP2801_CPLD_INT_RAW_REG	IXDP2801_CPLD_REG(0x10)	  /* read */
#define IXDP2801_CPLD_INT_MASK_CLR_REG	IXDP2801_CPLD_INT_RAW_REG /* write */
#define IXDP2801_CPLD_INT_SIM_REG	IXDP2801_CPLD_REG(0x14)
	/* Interrupt bits are defined in irqs.h */
#define IXDP2801_CPLD_GBE_REG		IXDP2801_CPLD_REG(0x18)
#define IXDP2801_CPLD_MISC_REG		IXDP2801_CPLD_REG(0x1C)
	#define IXDP2801_CPLD_MISC_REV_MASK 0xFF
#define IXDP2801_CPLD_FLASH_REG		IXDP2801_CPLD_REG(0x20)
	#define IXDP2801_CPLD_FLASH_INTERN 0x8000
	#define IXDP2801_CPLD_FLASH_BANK_MASK 0xF
#define IXDP2801_CPLD_FORK_REG		IXDP2801_CPLD_REG(0x30)

#define IXDP2801_FLASH_WINDOW_BITS 25
#define IXDP2801_FLASH_WINDOW_SIZE (1 << IXDP2801_FLASH_WINDOW_BITS)
#define IXDP2801_FLASH_WINDOW_MASK (IXDP2801_FLASH_WINDOW_SIZE - 1)

#ifndef __ASSEMBLY__
static inline unsigned long ixdp2801_set_flash_bank(unsigned long ofs)
{
	*IXDP2801_CPLD_FLASH_REG =
	    ((ofs >> IXDP2801_FLASH_WINDOW_BITS) |
	     IXDP2801_CPLD_FLASH_INTERN);
	return (ofs & IXDP2801_FLASH_WINDOW_MASK);
}

static inline unsigned long ixdp2801_get_flash_banks(void)
{
	*IXDP2801_CPLD_FLASH_REG =
	    (IXDP2801_CPLD_FLASH_BANK_MASK | IXDP2801_CPLD_FLASH_INTERN);
	return ((*IXDP2801_CPLD_FLASH_REG & IXDP2801_CPLD_FLASH_BANK_MASK)
		+ 1);
}
#endif

#endif /*_IXDP2801_H_ */

