/*
 * (C) Copyright 2001, 2002
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 *
 * (C) Copyright 2002
 * Frank Panno <fpanno@delphintech.com>, Delphin Technology AG
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <ioports.h>
#include <mpc8260.h>
#include <pci.h>

/*
 * I/O Port configuration table
 *
 * if conf is 1, then that port pin will be configured at boot time
 * according to the five values podr/pdir/ppar/psor/pdat for that entry
 */
/* conf = Specifies if this pin is to be configured.
   ppar = Pin Assignment Register,     0 - GPIO,                   1 - Dedicated Peripheral Function
   psor = Special Operations Register, 0 - Option 1,               1 - Option 2
   pdir = Data Direction Register,     0 - input or bidirectional, 1 - output
   podr = Open Drain Register,         0 - Actively Driven,        1 - Open Drain
   pdat = Data Register
   NA = Not Available

   SMC1        - UART
   SMC2        - Not Used
   SCC1        - UART Console
   SCC3/USB    - 
   SCC4        - Not Used
   FCC1        - Ethernet 1
   FCC2        - Ethernet 2
   SPI        - Not Used
   I2C        - I2C

   PPARA = 0x0003FC3F, PSORA = 0x0000003F, PDIRA = 0x00003C0C, PODRA = 0x00000000
*/
const iop_conf_t iop_conf_tab[4][32] = {
    /* Port A configuration */
    {    /*       conf ppar psor pdir podr pdat */
    /* PA31 */ {   1,   1,   1,   0,   0,   0   }, /* FCC1: COL */
    /* PA30 */ {   1,   1,   1,   0,   0,   0   }, /* FCC1: CRS */
    /* PA29 */ {   1,   1,   1,   1,   0,   0   }, /* FCC1: TX_ER */
    /* PA28 */ {   1,   1,   1,   1,   0,   0   }, /* FCC1: TX_EN */
    /* PA27 */ {   1,   1,   1,   0,   0,   0   }, /* FCC1: RX_DV */
    /* PA26 */ {   1,   1,   1,   0,   0,   0   }, /* FCC1: RX_ER */
    /* PA25 */ {   0,   0,   0,   0,   0,   0   }, /* PA25 */
    /* PA24 */ {   0,   0,   0,   0,   0,   0   }, /* PA24 */
    /* PA23 */ {   0,   0,   0,   0,   0,   0   }, /* PA23 */
    /* PA22 */ {   0,   0,   0,   0,   0,   0   }, /* PA22 */
    /* PA21 */ {   1,   1,   0,   1,   0,   0   }, /* FCC1: TxD[3] */
    /* PA20 */ {   1,   1,   0,   1,   0,   0   }, /* FCC1: TxD[2] */
    /* PA19 */ {   1,   1,   0,   1,   0,   0   }, /* FCC1: TxD[1] */
    /* PA18 */ {   1,   1,   0,   1,   0,   0   }, /* FCC1: TxD[0] */
    /* PA17 */ {   1,   1,   0,   0,   0,   0   }, /* FCC1: RxD[0] */
    /* PA16 */ {   1,   1,   0,   0,   0,   0   }, /* FCC1: RxD[1] */
    /* PA15 */ {   1,   1,   0,   0,   0,   0   }, /* FCC1: RxD[2] */
    /* PA14 */ {   1,   1,   0,   0,   0,   0   }, /* FCC1: RxD[3] */
    /* PA13 */ {   0,   0,   0,   0,   0,   0   }, /* PA13 */
    /* PA12 */ {   0,   0,   0,   0,   0,   0   }, /* PA12 */
    /* PA11 */ {   0,   0,   0,   0,   0,   0   }, /* PA11 */
    /* PA10 */ {   0,   0,   0,   0,   0,   0   }, /* PA10 */
    /* PA9  */ {   0,   0,   0,   0,   0,   0   }, /* PA9 */
    /* PA8  */ {   0,   0,   0,   0,   0,   0   }, /* PA8 */
    /* PA7  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PA6  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PA5  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PA4  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PA3  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PA2  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PA1  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PA0  */ {   0,   0,   0,   0,   0,   0   }  /* NA */
    },

/* PPARB =  0x00003FFF, PSORB = 0x00000004, PDIRB = 0x000003C5, PODRB = 0x00000000 */
    /* Port B configuration */
    {   /*      conf ppar psor pdir podr pdat */
    /* PB31 */ {   1,   1,   0,   1,   0,   0   }, /* FCC2: TX_ER */
    /* PB30 */ {   1,   1,   0,   0,   0,   0   }, /* FCC2: RX_DV */
    /* PB29 */ {   1,   1,   1,   1,   0,   0   }, /* FCC2: TX_EN */
    /* PB28 */ {   1,   1,   0,   0,   0,   0   }, /* FCC2: RX_ER */
    /* PB27 */ {   1,   1,   0,   0,   0,   0   }, /* FCC2: COL */
    /* PB26 */ {   1,   1,   0,   0,   0,   0   }, /* FCC2: CRS */
    /* PB25 */ {   1,   1,   0,   1,   0,   0   }, /* FCC2: TxD[3] */
    /* PB24 */ {   1,   1,   0,   1,   0,   0   }, /* FCC2: TxD[2] */
    /* PB23 */ {   1,   1,   0,   1,   0,   0   }, /* FCC2: TxD[1] */
    /* PB22 */ {   1,   1,   0,   1,   0,   0   }, /* FCC2: TxD[0] */
    /* PB21 */ {   1,   1,   0,   0,   0,   0   }, /* FCC2: RxD[0] */
    /* PB20 */ {   1,   1,   0,   0,   0,   0   }, /* FCC2: RxD[1] */
    /* PB19 */ {   1,   1,   0,   0,   0,   0   }, /* FCC2: RxD[2] */
    /* PB18 */ {   1,   1,   0,   0,   0,   0   }, /* FCC2: RxD[3] */
    /* PB17 */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PB16 */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PB15 */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PB14 */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PB13 */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PB12 */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PB11 */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PB10 */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PB9  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PB8  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PB7  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PB6  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PB5  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PB4  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PB3  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PB2  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PB1  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PB0  */ {   0,   0,   0,   0,   0,   0   }  /* NA */
    },

/* PPARC = 0x0C333E80, PSORC = 0x00000000, PDIRC = 0x04000800, PODRC = 0x00000000 */
    /* Port C */
    {   /*       conf ppar psor pdir podr pdat */
    /* PC31 */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PC30 */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PC29 */ {   0,   0,   0,   0,   0,   0   }, /* PC29 */
    /* PC28 */ {   0,   0,   0,   0,   0,   0   }, /* PC28 */
    /* PC27 */ {   0,   0,   0,   0,   0,   0   }, /* PC27 */
    /* PC26 */ {   0,   0,   0,   0,   0,   0   }, /* PC26 */
    /* PC25 */ {   0,   0,   0,   0,   0,   0   }, /* PC25 */
    /* PC24 */ {   0,   0,   0,   0,   0,   0   }, /* PC24 */
    /* PC23 */ {   0,   0,   0,   0,   0,   0   }, /* PC23 */
    /* PC22 */ {   1,   1,   0,   0,   0,   0   }, /* FCC1: TX_CLK10 */
    /* PC21 */ {   1,   1,   0,   0,   0,   0   }, /* FCC1: RX_CLK11 */
    /* PC20 */ {   0,   0,   0,   0,   0,   0   }, /* PC20 */
    /* PC19 */ {   1,   0,   0,   1,   0,   1   }, /* PHY: MDC */
    /* PC18 */ {   1,   0,   0,   1,   0,   1   }, /* PHY: MDIO */
    /* PC17 */ {   1,   1,   0,   0,   0,   0   }, /* FCC2: RX_CLK15 */
    /* PC16 */ {   1,   1,   0,   0,   0,   0   }, /* FCC2: TX_CLK16 */
    /* PC15 */ {   0,   1,   0,   0,   0,   0   }, /* SCC1: _CTS */
    /* PC14 */ {   0,   0,   0,   0,   0,   0   }, /* PC14 */
    /* PC13 */ {   0,   0,   0,   0,   0,   0   }, /* PC13 */
    /* PC12 */ {   0,   0,   0,   0,   0,   0   }, /* PC12 */
    /* PC11 */ {   0,   0,   0,   0,   0,   0   }, /* PC11 */
    /* PC10 */ {   0,   0,   0,   0,   0,   0   }, /* PC10 */
    /* PC9  */ {   0,   0,   0,   0,   0,   0   }, /* PC9 */
    /* PC8  */ {   0,   0,   0,   0,   0,   0   }, /* PC8 */
    /* PC7  */ {   0,   0,   0,   0,   0,   0   }, /* PC7 */
    /* PC6  */ {   0,   0,   0,   0,   0,   0   }, /* PC6 */
    /* PC5  */ {   0,   0,   0,   0,   0,   0   }, /* PC5 */
    /* PC4  */ {   0,   0,   0,   0,   0,   0   }, /* PC4 */
    /* PC3  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PC2  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PC1  */ {   0,   0,   0,   0,   0,   0   }, /* PC1 */
    /* PC0  */ {   0,   0,   0,   0,   0,   0   }, /* PC0 */
    },

/* PPARD = 0x000301C7, PSORD = 0x00030002, PDIRD = 0x00000186, PODRD = 0x00000000 */
    /* Port D */
    {   /*      conf ppar psor pdir podr pdat */
    /* PD31 */ {   1,   1,   0,   0,   0,   0   }, /* SCC1: RXD */
    /* PD30 */ {   1,   1,   1,   1,   0,   0   }, /* SCC1: TXD */
    /* PD29 */ {   0,   1,   0,   1,   0,   0   }, /* SCC1: _RTS */
    /* PD28 */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PD27 */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PD26 */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PD25 */ {   0,   0,   0,   0,   0,   0   }, /* PC25 */
    /* PD24 */ {   0,   0,   0,   0,   0,   0   }, /* PC24 */
    /* PD23 */ {   0,   0,   0,   0,   0,   0   }, /* SCC1:  */
    /* PD22 */ {   0,   0,   0,   0,   0,   0   }, /* PD22 */
    /* PD21 */ {   0,   0,   0,   0,   0,   0   }, /* PD21 */
    /* PD20 */ {   0,   0,   0,   0,   0,   0   }, /* PD20 */
    /* PD19 */ {   0,   0,   0,   0,   0,   0   }, /* PD19 */
    /* PD18 */ {   0,   0,   0,   0,   0,   0   }, /* PD18 */
    /* PD17 */ {   0,   0,   0,   0,   0,   0   }, /* PD17 */
    /* PD16 */ {   0,   0,   0,   0,   0,   0   }, /* PD16 */
    /* PD15 */ {   1,   1,   1,   0,   0,   0   }, /* I2C: SDA */
    /* PD14 */ {   1,   1,   1,   0,   0,   0   }, /* I2C: SCL */
    /* PD13 */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PD12 */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PD11 */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PD10 */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PD9  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PD8  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PD7  */ {   0,   0,   0,   0,   0,   0   }, /* PD7 */
    /* PD6  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PD5  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PD4  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PD3  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PD2  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PD1  */ {   0,   0,   0,   0,   0,   0   }, /* NA */
    /* PD0  */ {   0,   0,   0,   0,   0,   0   }  /* NA */
    }
};

/* ------------------------------------------------------------------------- */

/*
    Early Board Specific Initialization
*/
int board_early_init_f (void)
{
#if 0
    volatile immap_t *immap = (immap_t *) CFG_IMMR;
    volatile memctl8260_t *memctl = &immap->im_memctl;
#endif

    return 0;
}

/*
 * Check Board Identity:
 * I don' know, how the next board revisions will be coded.
 * Thats why its a static interpretation ...
 */

int checkboard (void)
{
    uint major = 0, minor = 0;

    printf ("Board: Vesuvius, Revision %d.%d\n", major, minor);
    return 0;
}


/* ------------------------------------------------------------------------- */

#define NOP  0x00000000 /* 000 Normal operation */
#define PAB  0x28000000 /* 101 Precharge all banks */
#define CBR  0x08000000 /* 001 CBR refresh */
#define MRW  0x18000000 /* 011 Mode register write */
#define RFEN 0x40000000 /* Refresh enable */

long int initdram (int board_type)
{
    long int msize = (128 * 1024 * 1024);
    volatile immap_t *immap = (immap_t *) CFG_IMMR;
    volatile memctl8260_t *memctl = &immap->im_memctl;

#ifndef CFG_RAMBOOT
    volatile uchar *ramaddr, c = 0xff;
    volatile unsigned long int *memptr;
    unsigned long int psdmr;

    unsigned long int i;

    immap->im_siu_conf.sc_ppc_acr = 0x00000002;
    immap->im_siu_conf.sc_ppc_alrh = 0x01234567;
    /* immap->im_siu_conf.sc_tescr1   = 0x00004000; */

    memctl->memc_psrt = CFG_PSRT;
    memctl->memc_mptpr = CFG_MPTPR;

#define PAGE_BASED

#ifdef BANK_BASED
    /* SDAM=001, BSMA=001, SDA10=011
       0000 0001 0010 1110 1011 1000 0000 0011 */
    psdmr = 0x012EB803; /* Bank-Based Interleaving */

    /* BRD=01, ROWST=0010, NUMR=100
       1111 1000 0000 0000 0010 0101 0000 0000 */
    memctl->memc_or2 = 0xF8002500;
#endif

#ifdef PAGE_BASED
 #if 0
    /* SDAM=011, BSMA=001, SDA10=011
       1000 0011 0010 1110 1011 1000 0000 0011 */
    psdmr = 0x832EB803; /* Page-Based Interleaving Slower Timing */
 #else
    /* 1000 0011 0010 1110 0010 0100 0000 0010 */
    psdmr = 0x832E2402; /* Page-Based Interleaving Faster Timing */
 #endif

    /* BRD=01, ROWST=0101, NUMR=100
       1111 1000 0000 0000 0010 1011 0000 0000 */
    memctl->memc_or2 = 0xF8002B00;
#endif

    memctl->memc_br2 = CFG_SDRAM_BASE | CFG_BR2;

    ramaddr = (uchar *) CFG_SDRAM_BASE;

    memctl->memc_psdmr = psdmr | NOP;    /* NOP */
    *(ramaddr + 0x0) = 0;

    memctl->memc_psdmr = psdmr | PAB;    /* Precharge all banks */
    *(ramaddr + 0x0) = 0;

    memctl->memc_psdmr = psdmr | CBR;    /* CBR refresh */
    *(ramaddr + 0x0) = 0;
    memctl->memc_psdmr = psdmr | NOP;    /* NOP */
    *(ramaddr + 0x0) = 0;

    memctl->memc_psdmr = psdmr | CBR;    /* CBR refresh */
    *(ramaddr + 0x0) = 0;
    memctl->memc_psdmr = psdmr | NOP;    /* NOP */
    *(ramaddr + 0x0) = 0;

    memctl->memc_psdmr = psdmr | MRW;    /* Mode Register write */
    *(ramaddr + 0x0) = 0;

    memctl->memc_psdmr = psdmr | NOP | RFEN;    /* NOP, Refresh enable */

    memptr = (unsigned long int *)(0);
    *(memptr) = 0;
    
    /* Autosize the amount of onboard memory.
     * This allows uboot to still operation if a reduced amount of memory is present.
     */
    for(i = 0x1000; i < msize; i <<= 1) {
        memptr = (unsigned long int *)(i);
        *(memptr) = i;
        if(*(memptr) != i)
            break;
        memptr = (unsigned long int *)(0);
        if(*(memptr) != 0)
            break;
    }
    msize = i;
    /* printf("Detected Memory  = %ld, 0x%lx\n", i, i); */

#endif /* CFG_RAMBOOT */

    memctl->memc_secmr = 0xFFFE0000;            /* SEC Mask Register - 128KB space */
    memctl->memc_secbr = CFG_IMMR + 0x40000;    /* SEC Base Resister */

    /* return total 60x bus SDRAM size in bytes */
    return msize;
}

#ifdef CONFIG_PCI
struct pci_controller hose;

extern void pci_mpc8250_init(struct pci_controller *);

void pci_init_board(void)
{
    pci_mpc8250_init(&hose);
}
#endif
