/*
 * (C) Copyright 2000-2002
 * Wolfgang Denk, DENX Software Engineering, wd@denx.de.
 * Rob Taylor, Flying Pig Systems. robt@flyingpig.com
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

#include <common.h>
#include <mpc824x.h>
#include <asm/processor.h>
#include <asm/pci_io.h>
#include <commproc.h>
#include "drivers/epic.h"

#if defined(CONFIG_GSM7248)
extern struct SrcVecTable SrcVecTable[MAXVEC];
#endif

int interrupt_init_cpu (unsigned *decrementer_count)
{
	*decrementer_count = (get_bus_freq (0) / 4) / CFG_HZ;

	/*
	 * It's all broken at the moment and I currently don't need
	 * interrupts. If you want to fix it, have a look at the epic
	 * drivers in dink32 v12. They do everthing and Motorola said
	 * I could use the dink source in this project as long as
	 * copyright notices remain intact.
	 */

#if defined(CONFIG_GSM7248)
	epicInit (EPIC_SERIAL_IRQ, 7); /* adopt serial mode interrupt with slowest clock --roger */
#else
	epicInit (EPIC_DIRECT_IRQ, 0);
#endif
	/* EPIC won't generate INT unless Current Task Pri < 15 */
	epicCurTaskPrioSet(0);

	return (0);
}

/****************************************************************************/
/****************************************************************************
* Interrupt Service Routines for GSM7248
*/
void GSM7248_ISR(unsigned long vector)
{
	/* Our purpose of ISR to handle all interrupts are to print out the type of interrupt.
	    It is not neccessary in this stage to hook one function for each interrupt. */

	if(vector > 0 || vector <=15) {
	/* print out the interrupt source name based on vector */
		printf("EPIC: %s\n", SrcVecTable[vector].srcName);
		switch (vector) {
			case 1:
			case 2:
			case 3:
			case 4:
			case 5:
			case 6:
			case 7:
			case 8:
			case 9:
			case 10:
			case 11:
				/* For testing, these interrupts are turnned on by EPIC test command 
				and turnned off here as soon as first interrupt are recevied */
				epicIntDisable(vector);
				break;
			default:
				break;
		}
		
				
	} else
		printf("Uknown Interrupt!\n");
	
}
/*
 * Handle external interrupts
 */
void external_interrupt (struct pt_regs *regs)
{
	register unsigned long vector;

	pci_readl (CFG_EUMB_ADDR + EPIC_PROC_INT_ACK_REG, vector);
	sync ();					/* i'm not convinced this is needed, but dink source has it */
	vector &= 0xff;				/*get vector */

	/*TODO: handle them -... */

#if defined(CONFIG_GSM7248)
	GSM7248_ISR(vector);
#endif
	epicEOI ();
}

/****************************************************************************/

/*
 * blank int handlers.
 */

void
irq_install_handler (int vec, interrupt_handler_t * handler, void *arg)
{
}

void irq_free_handler (int vec)
{

}

/*TODO: some handlers for winbond and 87308 interrupts
 and what about generic pci inteerupts?
 vga?
 */

void timer_interrupt_cpu (struct pt_regs *regs, ulong timestamp)
{
	/* nothing to do here */
	return;
}
